;; Generated package description from trust-manager.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "trust-manager" "0.4.2" "Convenient trust management" '((emacs "30.1")) :commit "0ff311bb59c9f1bb1ecdf413fa2bfa353bf389d7" :authors '(("Eshel Yaron" . "me@eshelyaron.com")) :maintainer '("Eshel Yaron" . "me@eshelyaron.com") :keywords '("security" "trust" "files") :url "https://git.sr.ht/~eshel/trust-manager")
